package sim.app.tutorial4;
import sim.engine.*;
import sim.field.grid.*;
import sim.util.*;
import ec.util.*;

public class Tutorial4 extends SimState
    {
    public DoubleGrid2D trails;
    public SparseGrid2D particles;

    public int collisions;
    public double collisionRate;
    public double getCollisionRate() { return collisionRate; }
    
    public int gridWidth = 100;
    public int gridHeight = 100;
    public int numParticles = 500;
    
    public int getWidth() { return gridWidth; }
    public void setWidth(int val) { if (val > 0 ) gridWidth = val; }
    public int getHeight() { return gridHeight; }
    public void setHeight(int val) { if (val > 0 ) gridHeight = val; }
    public int getNumParticles() { return numParticles; }
    public void setNumParticles(int val) { if (val >= 0) numParticles = val; }

    public Tutorial4(long seed)
        {
        super(new MersenneTwisterFast(seed), new Schedule(3));
        }

    public void start()
        {
        super.start();
        trails = new DoubleGrid2D(gridWidth, gridHeight);
        particles = new SparseGrid2D(gridWidth, gridHeight);
        
        Particle p;
        
        for(int i=0 ; i<numParticles ; i++)
            {
            p = new Particle(random.nextInt(3) - 1, random.nextInt(3) - 1);  // random direction
            schedule.scheduleRepeating(p);
            particles.setObjectLocation(p,
                                        new Int2D(random.nextInt(gridWidth),random.nextInt(gridHeight)));  // random location
            }
        
        // Schedule the "Big Particle"
        BigParticle b = new BigParticle(random.nextInt(3) - 1, random.nextInt(3) - 1);
        particles.setObjectLocation(b,
                                    new Int2D(random.nextInt(gridWidth),random.nextInt(gridHeight)));
        schedule.scheduleRepeating(Schedule.EPOCH,1,b,5);
        
        // Schedule the decreaser
        Steppable decreaser = new Steppable()
            {
            public void step(SimState state)
                {
                // decrease the trails
                trails.multiply(0.9);
                
                // compute and reset the collision info
                collisionRate = collisions / (double)numParticles;
                collisions = 0;
                }
                
            // anonymous class -- here's the serialVersionUID (see tutorial3)
            // by the way, notice it's DIFFERENT from tutorial3's due to new names of stuff.
            // No biggie -- as long as you have <i>some</i> serialVersionUID...
            static final long serialVersionUID = 6976157378487763326L;
            };
            
        schedule.scheduleRepeating(Schedule.EPOCH,2,decreaser,1);
        }

    public static void main(String[] args)
        {
        Tutorial4 tutorial4 = null;
        
        // should we load from checkpoint?  I wrote this little chunk of code to
        // check for this to give you the general idea.
        
        for(int x=0;x<args.length-1;x++)  // "-checkpoint" can't be the last string
            if (args[x].equals("-checkpoint"))
                {
                SimState state = SimState.readFromCheckpoint(new java.io.File(args[x+1]));
                if (state == null)   // there was an error -- it got printed out to the screen, so just quit
                    System.exit(1);
                else if (!(state instanceof Tutorial4))  // uh oh, wrong simulation stored in the file!
                    {
                    System.out.println("Checkpoint contains some other simulation: " + state);
                    System.exit(1);
                    }
                else // we're ready to lock and load!  
                    tutorial4 = (Tutorial4)state;
                }
        
        // ...or should we start fresh?
        if (tutorial4==null)  // no checkpoint file requested
            {
            tutorial4 = new Tutorial4(System.currentTimeMillis());
            tutorial4.start();
            }
        
        long time;
        while((time = tutorial4.schedule.time()) < 100)
            {
            if (time % 10 == 0) System.out.println(time);
            if (!tutorial4.schedule.step(tutorial4))
                break;

            // checkpoint
            if (time%500==0 && time!=0)
                {
                String s = "tutorial4." + time + ".checkpoint";
                System.out.println("Checkpointing to file: " + s);
                tutorial4.writeToCheckpoint(new java.io.File(s));
                }
            }
            
        tutorial4.finish();  // we don't use this, but it's good style
        }
    
    // contains an anonymous class -- here's the serialVersionUI (see tutorial3)
    // by the way, notice it's DIFFERENT from tutorial3's due to new names of stuff.
    // No biggie -- as long as you have <i>some</i> serialVersionUID...
    static final long serialVersionUID = 6930440709111220430L;
    }
